%===================================================================
%% Script file:
% This program solves the equations of linear multi-degree of freedom systems 
% using the algorthm of Table 9.1 in the book by John T. Katsikadelis
% "The Boundary Element Method for Engineers and Scientistss", Academic Press,
% Elsevier, 2016.Details in: 
% J.T.Katsikadelis. A new direct time integration method for the equations 
%of motion in structural dynamics, ZAMM  Z. Angew.Math.Mech.94,No.9,757774
%%
%  Record of revisions:
%      Date       Programmer         Description of change
%      ====       ==========         =====================
%   17/02/2012    J.T.Katsikadelis    Original code
%
clear all;
% =======================================================================
%% Input of data
% =======================================================================
N =100;            % number of unknowns = number of degrees of freedom =
             %     % number of equations
t_total=5;         % total time of motion 
h=0.01;            % time step ?t
t=h:h:t_total;     % time instances
nmax=length(t);    % number of steps
%------------------------------------------------------------------------
%%  Read Mass matrix M, Damping matrix C, Stiffness matrix K
% ---------------------------------------------------------------------
M = 10*rand(N);    % creates randomly a mass matrix
M=M'*M;            % this makes the random matrix symmetric. Not necessary
C =0*rand(N);      % creates randomly a damping matrix
C=C'*C;            % this makes the random matrix symmetric. Not necessary
K =10*rand(N);     % creates randomly a stiffeness matrix
K=K'*K;            % this makes the random matrix symmetric. Not necessary
%-------------------------------------------------------------------------
%%  Read the initial conditions u0, du0
% -------------------------------------------------------------------------
u0=ones(N,1);      % initial dispacement   
du0=5*ones(N,1);   % initial velocity
%%-------------------------------------------------------------------------
%% Read the excitation force
% ------------------------------------------------------------------------
p=zeros(N,nmax);   % excitation force
p0=0*ones(N,1);    % excitation force at t=0
p=0*ones(N,nmax);  % excitation force at t>0

tic   % counts execution time -start

%% ========================================================================
%Solution
% ========================================================================
c1=h^2/2; c2=h;  %
G=[ M C K
    c1*eye(N)/2 -h*eye(N)  eye(N)
    -c2*eye(N)/2 eye(N) zeros(N)];
H=[zeros(N) zeros(N) zeros(N)
    -c1*eye(N)/2 zeros(N) eye(N)
    c2*eye(N)/2 eye(N) zeros(N)];
B=[eye(N); zeros(N); zeros(N)];
Ginv=inv(G);
A=Ginv*H;
b=Ginv*B;
q0=inv(M)*(p0-C*du0-K*u0);
U0=[q0; du0; u0];    % intial solution vector, t=0
U=zeros(3*N,nmax);   % solution  vector t=h,2h, ....nmax*h
U(:,1)=A*U0+b*p(:,1);
for i=2:nmax
    U(:,i)=A*U(:,i-1)+b*p(:,i);
end
toc % counts execution time -end
%
t=[0 t];
U=[U0 U]; % solution vector t=0,h,2h, ....nmax*h  
% 
u=U(2*N+1:3*N,:);   % displacement
du=U(N+1:2*N,:);    % velocity
ddu=U(1:N,:);       % acceleration
%% =========================================================================
% Plot solution
% =========================================================================
figure(1);plot(t,u(2,:),'r-','LineWidth',2); title( 'displacement  u(2,t)') ;
grid;
figure(2);plot(t,du(2,:),'b-','LineWidth',2); title( 'velocity du(2,t)') ;
grid;


